<h2>pywb Sample Home Page</h2>

The following archive collections are available:

<ul>
{% for route in routes %}
  {% if route | is_wb_handler %}
  <li><a href="{{ '/' + route.path }}">{{ '/' + route.path }}</a>: {{ route | string }}</li>
  {% endif %} 
{% endfor %}
</ul>

Other endpoints in this deployment:

<ul>
{% for route in routes %}
  {% if not route | is_wb_handler %}
  <li><b>{{ '/' + route.path }}</b> - {{ route | string }}</li>
  {% endif %} 
{% endfor %}
</ul>

