<!-- WB Insert -->
{% if rule.js_rewrite_location != 'urls' and include_wombat %}
<script src='{{ wbrequest.host_prefix }}/{{ static_path }}/wombat.js'> </script>
<script>
  {% set urlsplit = cdx.original | urlsplit %}
  if (window && window._WBWombat && !window._wb_js_inited) {
  var _wb_wombat = new _WBWombat("{{ wbrequest.wb_prefix}}",
                 "{{ cdx['timestamp'] if include_ts else ''}}",
                 "{{ urlsplit.scheme }}",
                 "{{ urlsplit.netloc }}",
                 "{{ cdx.timestamp | format_ts('%s') }}",
                 "{{ wbrequest.wb_url.mod }}");
  }
</script>
{% endif %}
<script>
  wbinfo = {}
  wbinfo.url = "{{ cdx.original }}";
  wbinfo.timestamp = "{{ cdx.timestamp }}";
  wbinfo.prefix = "{{ wbrequest.wb_prefix }}";
  wbinfo.mod = "{{ wbrequest.wb_url.mod }}";
  wbinfo.top_url = "{{ top_url }}";
  wbinfo.is_live = {{ "true" if cdx.is_live else "false" }};
  wbinfo.coll = "{{ wbrequest.coll }}";
  wbinfo.proxy_magic = "{{ wbrequest.env.pywb_proxy_magic }}";
  wbinfo.static_prefix = "{{ wbrequest.host_prefix }}/{{ static_path }}";
</script>

<script src='{{ wbrequest.host_prefix }}/{{ static_path }}/wb.js'> </script>
<script src='{{ wbrequest.host_prefix }}/{{ static_path }}/vidrw.js'> </script>

{% include banner_html ignore missing %}

<!-- load banner -->
<script> if (_wb_js) { _wb_js.load(); }</script>

<!-- End WB Insert -->

