
#_wb_plain_banner, #_wb_frame_top_banner
{ 
    display: block !important;
    top: 0px !important;
    left: 0px !important;
    font-family: "Open Sans", "Helvetica Neue", Helvetica, Arial, sans-serif !important;
    width: 100% !important;
    font-size: 24px !important;
    background-color: lightYellow !important; 
    color: black !important;
    text-align: center !important;
    z-index: 2147483643 !important;
    line-height: normal !important;
}

#_wb_plain_banner
{
    position: absolute !important;
    padding: 4px !important;
    border: 1px solid !important;
}

#_wb_frame_top_banner
{
    position: fixed !important;
    border: 0px;
    height: 40px !important;
}

.wb_iframe_div
{
    width: 100%;
    height: 100%;
    padding: 40px 4px 4px 0px;
    border: none;
    box-sizing: border-box;
    -moz-box-sizing: border-box;
    -webkit-box-sizing: border-box;
}

.wb_iframe
{
    width: 100%;
    height: 100%;
    border: 2px solid tan;
}
